import pytest
from notter.utils import convert_to_local_path


class TestUtils:
    @pytest.mark.parametrize(
        "filepath, src_path, expected",
        [
            ("/home/user/notes.py", "/home/user/notes", ".py"),
            ("/home/user/notes.py", "/home/user", "notes.py"),
            ("/home/user/notes.py", "/home", "user/notes.py"),
            ("home/user/notes.py", "src_path_not_in", "home/user/notes.py"),
            ("/home/user/notes.py", "src_path_not_in", "home/user/notes.py"),
        ],
    )
    def test_convert_to_local_path(self, filepath: str, src_path: str, expected: str) -> None:
        assert convert_to_local_path(filepath, src_path) == expected



import os
from pathlib import Path
import shutil
from typing import Generator
from notter.model import Content, Note, NoteType, NoteWithContent
from pytest import fixture
from notter.controller import NoteController

from notter.notter import Notter


@fixture
def content() -> Content:
    return Content("This is a note")


@fixture
def note() -> Note:
    return Note(
        id="note_id",
        username="pikachu",
        email="pikachu@test.com",
        filepath="path/to/file",
        line=1,
        type=NoteType.TODO,
        created_at="2021-01-01T00:00:00",
        updated_at="2021-02-01T10:00:00",
    )


@fixture
def note_with_content(note: Note, content: Content) -> NoteWithContent:
    return NoteWithContent(note, content)


@fixture
def temp_directory() -> Generator[Path, None, None]:
    test_dir: Path = Path(__file__).parent.resolve()
    tmp_dir: Path = test_dir / "tmp"

    os.makedirs(str(tmp_dir), exist_ok=True)
    yield tmp_dir
    shutil.rmtree(str(tmp_dir))


@fixture
def notter_with_config(temp_directory: Path) -> Notter:
    notter = Notter()
    notter.configure(temp_directory / "src")
    return notter


@fixture
def note_controller(notter_with_config) -> NoteController:
    return NoteController(notter_with_config)



from pathlib import Path
from typing import List
from unittest.mock import MagicMock, patch, mock_open
import notter.constants as ncons
from notter.index import NoteIndex
from notter.model import Comment, Content, Note, NoteType, NoteWithContent
from notter.notter import Notter
from notter.repository import JsonFileRepository


class TestJsonFileRepository:
    @patch("notter.index.NoteIndex.init_index")
    def test_initialize(self, init_idx_mock: MagicMock, notter_with_config: Notter) -> None:
        assert notter_with_config.config.get(ncons.NOTES_INDEX_PATH) is None
        assert notter_with_config.config.get(ncons.IDX_INITIALIZED_FLAG) is False

        repository = JsonFileRepository(notter_with_config)

        assert hasattr(repository, "note_index")
        assert isinstance(repository.note_index, NoteIndex)
        assert notter_with_config.config.get(ncons.NOTES_INDEX_PATH) is not None
        assert notter_with_config.config.get(ncons.IDX_INITIALIZED_FLAG) is True
        init_idx_mock.assert_called_once()

    @patch("notter.index.NoteIndex.load_index")
    def test_load(self, load_idx_mock: MagicMock, notter_with_config: Notter) -> None:
        notter_with_config.set_config(ncons.NOTES_INDEX_PATH, "some/path")
        notter_with_config.set_config(ncons.IDX_INITIALIZED_FLAG, True)

        repository = JsonFileRepository(notter_with_config)

        assert hasattr(repository, "note_index")
        assert isinstance(repository.note_index, NoteIndex)
        load_idx_mock.assert_called_once()

    @patch("notter.index.NoteIndex.store")
    @patch("notter.repository.JsonFileRepository._get_note_content_path")
    def test_create(
        self,
        get_content_path_mock: MagicMock,
        store_mock: MagicMock,
        notter_with_config: Notter,
        note_with_content: NoteWithContent,
    ) -> None:
        get_content_path_mock.return_value = "mock/path"
        notter_with_config.set_config(ncons.NOTES_INDEX_PATH, "some/path")
        notter_with_config.set_config(ncons.IDX_INITIALIZED_FLAG, False)
        repository = JsonFileRepository(notter_with_config)

        m_open = mock_open()
        with patch("builtins.open", m_open) as mock_file:
            repository.create(note_with_content)

        repository.note_index.store.assert_called_once_with(note_with_content.note)
        m_open.assert_called_with("mock/path", "w")

    @patch("notter.index.NoteIndex.fetch")
    @patch("notter.repository.JsonFileRepository._get_note_content_path")
    def test_read(self, get_content_path_mock: MagicMock, fetch_mock: MagicMock, notter_with_config: Notter) -> None:
        fetch_mock.return_value = Note(id="dummy_id", username="user", email="a@b.com", filepath="dummy_path", line=5)
        get_content_path_mock.return_value = "mock/path"
        notter_with_config.set_config(ncons.NOTES_INDEX_PATH, "some/path")
        notter_with_config.set_config(ncons.IDX_INITIALIZED_FLAG, False)
        repository = JsonFileRepository(notter_with_config)

        m_open = mock_open()
        with patch("builtins.open", m_open) as mock_file:
            fetched_note: NoteWithContent = repository.read("dummy_path", 5)

        assert fetched_note.note.id == "dummy_id"
        m_open.assert_called_with("mock/path", "r")

    @patch("notter.index.NoteIndex.store")
    @patch("notter.repository.JsonFileRepository.read")
    @patch("notter.repository.JsonFileRepository._get_note_content_path")
    def test_update(
        self, get_content_path_mock: MagicMock, read_mock: MagicMock, store_mock: MagicMock, notter_with_config: Notter
    ) -> None:
        dummy_note = Note(id="dummy_id", username="user", email="a@b.com", filepath="dummy_path", line=5)
        new_note = Note(id="dummy_id", username="pikachu", email="pika@chu.com", filepath="dummy_path", line=5)

        existing_note = NoteWithContent(dummy_note, Content("old content"))
        read_mock.return_value = existing_note
        get_content_path_mock.return_value = "mock/path"
        notter_with_config.set_config(ncons.NOTES_INDEX_PATH, "some/path")
        notter_with_config.set_config(ncons.IDX_INITIALIZED_FLAG, False)
        repository = JsonFileRepository(notter_with_config)

        m_open = mock_open()
        with patch("builtins.open", m_open) as mock_file:
            repository.update("dummy_path", 5, NoteWithContent(new_note, Content("new content")))

        m_open.assert_called_with("mock/path", "w")
        read_mock.assert_called_once_with("dummy_path", 5)
        store_mock.assert_called_once_with(existing_note.note, True)

    @patch("notter.index.NoteIndex.clean")
    @patch("notter.repository.JsonFileRepository._get_note_content_path")
    def test_delete(self, get_content_path_mock: MagicMock, clean_mock: MagicMock, notter_with_config: Notter) -> None:
        get_content_path_mock.return_value = "mock/path"
        clean_mock.return_value = Note(id="dummy_id", username="user", email="a@b.com", filepath="dummy_path", line=5)
        notter_with_config.set_config(ncons.NOTES_INDEX_PATH, "some/path")
        notter_with_config.set_config(ncons.IDX_INITIALIZED_FLAG, False)
        repository = JsonFileRepository(notter_with_config)

        with patch("os.unlink") as mock_unlink:
            repository.delete("dummy_path", 5)

        clean_mock.assert_called_once_with("dummy_path", "5")
        get_content_path_mock.assert_called_once_with("dummy_id")

    @patch("notter.index.NoteIndex.summarize")
    @patch("notter.repository.JsonFileRepository.delete")
    def test_prune(self, delete_mock: MagicMock, summarize_mock: MagicMock, notter_with_config: Notter) -> None:
        summarize_mock.return_value = set(["mock_path1:2", "mock_path2:3", "mock_path2:4"])
        notter_with_config.set_config(ncons.NOTES_INDEX_PATH, "some/path")
        notter_with_config.set_config(ncons.IDX_INITIALIZED_FLAG, False)
        repository = JsonFileRepository(notter_with_config)
        comments: List[Comment] = [
            Comment("mock_path1", "text", 2, NoteType.TODO),
            Comment("mock_path2", "texttext", 4, NoteType.NOTE),
        ]

        items_to_prune = repository.prune(comments)

        assert items_to_prune == ["mock_path2:3"]
        delete_mock.assert_called_once_with("mock_path2", 3)
        summarize_mock.assert_called_once_with()

    def test_get_note_content_path(self, notter_with_config: Notter) -> None:
        notter_with_config.set_config(ncons.NOTES_PATH, "note/path")
        notter_with_config.set_config(ncons.IDX_INITIALIZED_FLAG, False)
        repository = JsonFileRepository(notter_with_config)

        assert repository._get_note_content_path("dummy_id") == "note/path/dummy_id"



from dataclasses import asdict
from unittest.mock import ANY, MagicMock, mock_open, patch

import pytest
from notter.exceptions import NoteAlreadyExists, NoteNotFound
from notter.index import NoteIndex
from notter.model import Note
from notter.utils import CustomEncoder


class TestNoteIndex:
    mock_index_path = "path/mock_index.json"
    mock_filepath = "path/mock_file.txt"
    mock_line = "1"
    mock_note_idx_entry = {"filepath": mock_filepath, "line": mock_line}
    mock_note = Note(id="dummy_id", username="user", email="a@b.com", filepath=mock_filepath, line=mock_line)

    @patch("json.dump")
    def test_init_index(self, json_dump_mock: MagicMock) -> None:
        index = NoteIndex(self.mock_index_path)

        m_open = mock_open()
        with patch("builtins.open", m_open) as mock_file:
            index.init_index(self.mock_index_path)

        m_open.assert_called_with(self.mock_index_path, "w+")
        json_dump_mock.assert_called_once_with({}, ANY, cls=CustomEncoder, indent=4)

    @patch("json.loads")
    def test_load_index(self, json_loads_mock: MagicMock) -> None:
        index = NoteIndex(self.mock_index_path)

        m_open = mock_open()
        with patch("builtins.open", m_open) as mock_file:
            index.load_index(self.mock_index_path)

        m_open.assert_called_with(self.mock_index_path, "r")
        json_loads_mock.assert_called_once()

    def test_seek(self) -> None:
        index = NoteIndex(self.mock_index_path)
        index.idx = {self.mock_filepath: {self.mock_line: self.mock_note_idx_entry}}

        assert index.seek(self.mock_filepath, self.mock_line) is True

    def test_seek_nonexistent_file_entry(self) -> None:
        index = NoteIndex(self.mock_index_path)
        assert index.seek("path/mock_file.txt", "2") is False

    def test_seek_nonexistent_line_entry(self) -> None:
        index = NoteIndex(self.mock_index_path)
        index.idx = {self.mock_filepath: {self.mock_line: self.mock_note_idx_entry}}

        assert index.seek(self.mock_filepath, "2") is False

    @patch("notter.index.NoteIndex.seek")
    def test_fetch(self, seek_mock: MagicMock) -> None:
        seek_mock.return_value = True
        index = NoteIndex(self.mock_index_path)
        index.idx = {self.mock_filepath: {self.mock_line: asdict(self.mock_note)}}

        fetched_note = index.fetch(self.mock_filepath, self.mock_line)
        assert isinstance(fetched_note, Note)
        assert fetched_note.id == self.mock_note.id

    @patch("notter.index.NoteIndex.seek")
    def test_fetch_nonexistent(self, seek_mock: MagicMock) -> None:
        seek_mock.return_value = False
        index = NoteIndex(self.mock_index_path)

        with pytest.raises(NoteNotFound):
            index.fetch("path/mock_file.txt", "2")

    def test_summarize(self) -> None:
        index = NoteIndex(self.mock_index_path)
        index.idx = {self.mock_filepath: {self.mock_line: self.mock_note}}

        result = index.summarize()
        assert isinstance(result, set)
        assert len(result) == 1
        assert result.pop() == "path/mock_file.txt:1"

    def test_summarize_empty(self) -> None:
        index = NoteIndex(self.mock_index_path)
        index.idx = {}

        result = index.summarize()
        assert isinstance(result, set)
        assert len(result) == 0

    @patch("notter.index.NoteIndex.seek")
    def test_store(self, seek_mock: MagicMock) -> None:
        seek_mock.return_value = False
        index = NoteIndex(self.mock_index_path)
        index.idx = {}

        m_open = mock_open()
        with patch("builtins.open", m_open) as mock_file:
            index.store(self.mock_note)

        m_open.assert_called_with(self.mock_index_path, "w")
        assert index.idx == {self.mock_filepath: {self.mock_line: asdict(self.mock_note)}}

    @patch("notter.index.NoteIndex.seek")
    def test_store_existing(self, seek_mock: MagicMock) -> None:
        seek_mock.return_value = True
        index = NoteIndex(self.mock_index_path)

        with pytest.raises(NoteAlreadyExists):
            index.store(self.mock_note)

    @patch("notter.index.NoteIndex.fetch")
    def test_clean(self, fetch_mock: MagicMock) -> None:
        fetch_mock.return_value = self.mock_note
        index = NoteIndex(self.mock_index_path)
        index.idx = {self.mock_filepath: {self.mock_line: self.mock_note}}

        m_open = mock_open()
        with patch("builtins.open", m_open) as mock_file:
            return_val = index.clean(self.mock_filepath, self.mock_line)

        m_open.assert_called_with(self.mock_index_path, "w")
        assert index.idx == {}
        assert return_val == self.mock_note

    @patch("notter.index.NoteIndex.fetch")
    def test_clean_nonexistent(self, fetch_mock: MagicMock) -> None:
        fetch_mock.side_effect = NoteNotFound
        index = NoteIndex(self.mock_index_path)

        m_open = mock_open()
        with patch("builtins.open", m_open) as mock_file:
            return_val = index.clean("asfjhgask", self.mock_line)

        m_open.assert_called_with(self.mock_index_path, "w")
        assert return_val is None



from pathlib import Path
from unittest.mock import ANY, MagicMock, call, mock_open, patch

import pytest
from notter.notter import Notter


class TestNotter:
    mock_src_folder = Path("mock/notter/src")

    def test_notter(self) -> None:
        notter = Notter()

        assert notter.config.get("initialized") is False
        assert notter.config.get("idx_initialized") is False

    def test_notter_init(self) -> None:
        notter = Notter()
        notter.init_notter_folders = MagicMock()

        m_open = mock_open()
        with patch("builtins.open", m_open) as mock_file:
            notter.configure(self.mock_src_folder)

        assert notter.config.get("config_path") == "mock/notter/.notter/config.json"
        assert notter.config.get("src_path") == "mock/notter/src"
        assert notter.config.get("path") == "mock/notter/.notter"
        assert notter.config.get("notes_path") == "mock/notter/.notter/notes"
        notter.init_notter_folders.assert_called_once()
        m_open.assert_called_with("mock/notter/.notter/config.json", "w+")

    def test_notter_init_folders(self) -> None:
        notter = Notter()
        notter.set_config = MagicMock()
        notter.config = {"path": "mock/notter/.notter", "notes_path": "mock/notter/.notter/notes"}

        with patch("os.mkdir") as mkdir_mock:
            notter.init_notter_folders()
            mkdir_mock.call_count == 2

        notter.set_config.assert_called_once_with("initialized", True)

    def test_notter_init_already_initialized(self) -> None:
        notter = Notter()
        notter.set_config = MagicMock()
        notter.config = {"initialized": True, "path": "dummy_path"}

        with patch("os.mkdir") as mkdir_mock:
            notter.init_notter_folders()
            mkdir_mock.assert_not_called()

        notter.set_config.assert_not_called()

    @patch("notter.notter.load_config")
    def test_load(self, load_config_mock: MagicMock) -> None:
        notter = Notter()
        notter.config = {"path": "mock/notter/.notter"}
        load_config_mock.return_value = True

        notter.load(self.mock_src_folder)

        load_config_mock.assert_called_once()

    @patch("notter.notter.load_config")
    def test_load_not_initialized(self, load_config_mock: MagicMock) -> None:
        notter = Notter()
        notter.config = {"path": "mock/notter/.notter"}
        load_config_mock.return_value = False

        with pytest.raises(SystemExit):
            notter.load(self.mock_src_folder)

        load_config_mock.assert_called_once()

    @patch("notter.notter.rmtree")
    def test_destroy(self, rmtree_mock: MagicMock) -> None:
        notter = Notter()
        notter.config = {"path": "mock/notter/.notter", "initialized": True}

        notter.destroy()
        rmtree_mock.assert_called_once_with("mock/notter/.notter")

    def test_repr_config(self) -> None:
        notter = Notter()
        notter.config = {"path": "mock/notter/.notter", "initialized": True}

        assert notter.repr_config() == "Notter config:\n==============\npath: mock/notter/.notter\ninitialized: True"

    def test_get_config(self) -> None:
        notter = Notter()
        notter.config = {"path": "mock/notter/.notter", "initialized": True}

        assert notter.get_config("path") == "mock/notter/.notter"
        assert notter.get_config("initialized") is True

    def test_get_config_not_found(self) -> None:
        notter = Notter()
        notter.config = {"path": "mock/notter/.notter", "initialized": True}

        result = notter.get_config("not_found")
        assert result is None

    @patch("notter.notter.persist_config_after")
    def test_set_config(self, persist_config_mock: MagicMock) -> None:
        notter = Notter()
        notter.config = {"config_path": "mock/notter/.notter/config.json"}

        m_open = mock_open()
        with patch("builtins.open", m_open) as mock_file:
            notter.set_config("path", "new_path")

        m_open.assert_called_with("mock/notter/.notter/config.json", "w+")
        assert notter.config.get("path") == "new_path"



from click.testing import CliRunner
from notter.cli import cli

from importlib.metadata import version as pkg_version


class TestCLI:
    def test_cli_version(self) -> None:
        runner = CliRunner(env={"SRC_PATH": "."})
        result = runner.invoke(cli, ["--version"])
        assert result.exit_code == 0
        assert result.output.strip("\n") == pkg_version("notter")



from pathlib import Path

import notter.constants as ncons

from unittest.mock import MagicMock, call
from notter.controller import NoteController
from notter.exceptions import NoteAlreadyExists
from notter.model import Comment, NoteWithContent


class TestNoteController:
    def test_create_note_with_content(self, note_controller: NoteController) -> None:
        file_path = Path(note_controller.notter.get_config(ncons.SRC_PATH)) / "path/to/file.py"
        note_with_content = note_controller._create_note_with_content(str(file_path), 1, "This is a test", "NOTE")

        assert note_with_content.note.filepath == "path/to/file.py"
        assert note_with_content.note.line == 1
        assert note_with_content.content.text == "This is a test"
        assert note_with_content.note.type == "NOTE"
        assert isinstance(note_with_content, NoteWithContent)

    def test_create(self, note_controller: NoteController) -> None:
        mock_note = "pikachu"
        note_controller.repository = MagicMock()
        note_controller._create_note_with_content = MagicMock(return_value=mock_note)
        note_controller.create("path/to/file.py", 1, "This is a test", "NOTE")

        note_controller.repository.create.assert_called_once_with(mock_note)

    def test_read(self, note_controller: NoteController) -> None:
        note_controller.repository = MagicMock()
        note_controller.read("path/to/file.py", 1)

        note_controller.repository.read.assert_called_once_with("path/to/file.py", 1)

    def test_update(self, note_controller: NoteController) -> None:
        mock_note = "pikachu"
        note_controller.repository = MagicMock()
        note_controller._create_note_with_content = MagicMock(return_value=mock_note)
        note_controller.update("path/to/file.py", 1, "This is a test", "NOTE")

        note_controller.repository.update.assert_called_once_with("path/to/file.py", 1, mock_note)

    def test_delete(self, note_controller: NoteController) -> None:
        note_controller.repository = MagicMock()
        note_controller.delete("path/to/file.py", 1)

        note_controller.repository.delete.assert_called_once_with("path/to/file.py", 1)

    def test_discover(self, note_controller: NoteController) -> None:
        mock_comments = [
            Comment("path/to/file.py", "This is a note", 1, "NOTE"),
            Comment("path/to/file.py", "This is a todo", 2, "TODO"),
        ]

        note_controller.create = MagicMock()
        note_controller.repository = MagicMock()
        note_controller.explorer = MagicMock(discover=MagicMock(return_value=mock_comments))

        comments = note_controller.discover(["tag1", "tag2"])

        assert comments == mock_comments
        note_controller.repository.prune.assert_called_once_with(mock_comments)
        note_controller.create.assert_has_calls(
            [call("path/to/file.py", 1, "This is a note", "NOTE"), call("path/to/file.py", 2, "This is a todo", "TODO")]
        )

    def test_discover_no_save(self, note_controller: NoteController) -> None:
        mock_comments = [
            Comment("path/to/file.py", "This is a note", 1, "NOTE"),
            Comment("path/to/file.py", "This is a todo", 2, "TODO"),
        ]

        note_controller.create = MagicMock()
        note_controller.repository = MagicMock()
        note_controller.explorer = MagicMock(discover=MagicMock(return_value=mock_comments))

        comments = note_controller.discover(["tag1", "tag2"], save_as_notter_notes=False)

        assert comments == mock_comments
        note_controller.repository.prune.assert_called_once_with(mock_comments)
        note_controller.create.assert_not_called()

    def test_discover_duplicate_comment(self, note_controller: NoteController) -> None:
        mock_comments = [Comment("path/to/file.py", "This is a note", 1, "NOTE")]
        note_controller.create = MagicMock()
        note_controller.create.side_effect = NoteAlreadyExists
        note_controller.repository = MagicMock()
        note_controller.explorer = MagicMock(discover=MagicMock(return_value=mock_comments))

        comments = note_controller.discover(["tag1", "tag2"])

        assert comments == mock_comments
        note_controller.repository.prune.assert_called_once_with(mock_comments)
        note_controller.create.assert_called_once_with("path/to/file.py", 1, "This is a note", "NOTE")

    def test_discover_no_comments(self, note_controller: NoteController) -> None:
        mock_comments = []
        note_controller.create = MagicMock()
        note_controller.repository = MagicMock()
        note_controller.explorer = MagicMock(discover=MagicMock(return_value=mock_comments))

        comments = note_controller.discover(["tag1", "tag2"])

        assert comments == mock_comments
        note_controller.repository.prune.assert_called_once_with(mock_comments)
        note_controller.create.assert_not_called()

    def test_discover_no_tags(self, note_controller: NoteController) -> None:
        mock_comments = []
        note_controller.create = MagicMock()
        note_controller.repository = MagicMock()
        note_controller.explorer = MagicMock(discover=MagicMock(return_value=mock_comments))

        comments = note_controller.discover([])

        assert comments == mock_comments
        note_controller.repository.prune.assert_called_once_with(mock_comments)
        note_controller.create.assert_not_called()
